import django.contrib.auth.validators
from django.db import migrations, models


def populate_usernames(apps, schema_editor):
    User = apps.get_model('example', 'MyUser')
    for u in User.objects.all().iterator():
        if u.um_id:
            u.username = 'um_%s' % u.um_id
        else:
            u.username = 'user_%s' % u.id
        u.save()


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0008_auto_20180208_1019'),
    ]

    operations = [
        migrations.AddField(
            model_name='myuser',
            name='username',
            field=models.CharField(null=True,
                                   error_messages={'unique': 'A user with that username already exists.'},
                                   help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.',
                                   max_length=150, unique=True,
                                   validators=[django.contrib.auth.validators.UnicodeUsernameValidator()],
                                   verbose_name='username'),
        ),
        migrations.RunPython(populate_usernames),
        migrations.AlterField(
            model_name='myuser',
            name='username',
            field=models.CharField(error_messages={'unique': 'A user with that username already exists.'},
                                   help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.',
                                   max_length=150, unique=True,
                                   validators=[django.contrib.auth.validators.UnicodeUsernameValidator()],
                                   verbose_name='username'),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-11-10 14:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0005_auto_20171107_1628'),
    ]

    operations = [
        migrations.AlterField(
            model_name='myuser',
            name='customer_category',
            field=models.CharField(blank=True, choices=[('QUALITY_PARTNER', 'quality partner'), ('DEALER', 'dealer'), ('ARCHITECT_PLANNER', 'architect/planner'), ('HOUSING_SOCIETY', 'housing society'), ('PRIVATE_CUSTOMER', 'private customer'), ('OTHERS', 'others')], max_length=200, null=True, verbose_name='customer category'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='title',
            field=models.CharField(blank=True, choices=[('MR', 'Mr.'), ('MRS', 'Mrs.')], max_length=20, null=True, verbose_name='title'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-11-01 17:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0003_myuser_crm_contact_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='myuser',
            name='avatar_attachment_id',
            field=models.IntegerField(blank=True, null=True, verbose_name='avatar attachment id'),
        ),
        migrations.AddField(
            model_name='myuser',
            name='company_logo_attachment_id',
            field=models.IntegerField(blank=True, null=True, verbose_name='company logo attachment id'),
        ),
        migrations.AddField(
            model_name='myuser',
            name='um_id',
            field=models.IntegerField(blank=True, null=True, verbose_name='user management id'),
        ),
    ]



from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import PermissionsMixin, UserManager
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class AbstractSolarluxUser(AbstractBaseUser, PermissionsMixin):
    """
    An abstract base class implementing a fully featured User model with
    admin-compliant permissions.

    Username is required. Other fields are optional.
    """
    username_validator = UnicodeUsernameValidator()

    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=_('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(_('first name'), max_length=255, null=True, blank=True)
    last_name = models.CharField(_('last name'), max_length=255, null=True, blank=True)

    customer_no = models.CharField(_('customer no'), max_length=255, null=True, blank=True)
    customer_name = models.CharField(_('customer name'), max_length=255, null=True, blank=True)

    language = models.CharField(
        _('language'),
        max_length=255,
        null=True, blank=True)

    TITLE_CHOICES = (
        ('MR', _('Mr.')),
        ('MRS', _('Mrs.')),
    )

    title = models.CharField(_('title'), choices=TITLE_CHOICES, max_length=255, null=True, blank=True)

    is_customer_admin = models.BooleanField(
        _('is customer admin'),
        default=False,
    )

    crm_contact_id = models.UUIDField(
        _('CRM contact id'),
        null=True, blank=True
    )

    email = models.EmailField(_('email address'), blank=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    access_token = models.TextField('OAuth2 Access Token', null=True, blank=True)
    password = models.CharField(_('password'), max_length=128, null=True, blank=True)

    objects = UserManager()

    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def clean(self):
        super().clean()
        self.email = self.__class__.objects.normalize_email(self.email)

    def get_full_name(self):
        """
        Return the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        """Return the short name for the user."""
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this user."""
        send_mail(subject, message, from_email, [self.email], **kwargs)



import requests

from django.test import TestCase

# Create your tests here.
from slxauth.utils import get_authenticator, is_authorized


class AuthTestCase(TestCase):
    def test_staff_role_has_effect(self):
        authorities = ['ROLE_GESTELLVERWALTUNG_ADMIN', 'ROLE_CMS_EMPLOYEE', 'ROLE_ABC']
        required = ['ROLE_GESTELLVERWALTUNG_ADMIN']

        self.assertTrue(is_authorized(authorities, required))


class ParseTokenTestCase(TestCase):
    def setUp(self):
        self.auth = get_authenticator()

    def test_token_can_be_decoded(self):
        """Sample Authentication access_token can be correctly decoded"""

        token = 'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJsYXN0TmFtZSI6IkRvZSB0ZXN0IiwibGFuZ3VhZ2UiOiJlbiIsImlzQ3VzdG9tZXJBZG1pbiI6dHJ1ZSwiY3VzdG9tZXJOdW1iZXIiOiI1MDAwMTY3IiwidGl0bGUiOiJNUiIsImN1c3RvbWVyTmFtZSI6IkxpbmFyYSIsImF1dGhvcml0aWVzIjpbIlJPTEVfTkVXU19BUFAiLCJST0xFX1NQQUNFUyIsIlJPTEVfVklERU9TIiwiUk9MRV9HRU5FUkFMX05FV1MiLCJST0xFX1NPTEFSTFVYX0lOU0lERSIsIlJPTEVfVEVDSE5JQ0FMX0RPQ1MiLCJST0xFX1NPTEFSTFVYX0FETUlOIl0sImNsaWVudF9pZCI6Im15LWNsaWVudC13aXRoLXJlZ2lzdGVyZWQtcmVkaXJlY3QiLCJmaXJzdE5hbWUiOiJKb2huIiwic2NvcGUiOlsicmVhZCIsInRydXN0Il0sImlkIjoxMywiZXhwIjoxNTE1NTQ3OTY5LCJjcm1Db250YWN0SWQiOm51bGwsImVtYWlsIjoiam9obi5kb2UyQGV4YW1wbGUubmV0IiwianRpIjoiYjQ4NDQ0MTktYTFiNi00MGY4LWI3NTUtYTE3NzJiMjNjM2JiIn0.Gp7TR1McKE8KM-bPu1nvDMrKjvuC5Xe6CuVlPmiG-jUmlypgOl-_fonCo6OOozA90YXZPdq0n0FFt9a4oMjPXmQj00WMgOIvpH5yzRWNHArZQTH8vdAm4TU1iA2juzzz-WRQJz-khL4mvXDq24Ezcy7ZGYZ0sUZROGzWuEJi6IDcxpj659O1LWYpvXaVDocfPA5DFhrpbXQr3BePSgNAFrdm1sXOBXSRHNCeMbRVBPz1UDxRYGrXiuyU770uiVR8bbkOsS9QECgryHKpW6KO0q_xfYcEiuvOBAiaMHk7N4yALjYXq-NbRbonXVueLuKLd3lrI2_tdah96iOQ5JAh0A'

        token_data = self.auth.decode_token(token)
        user = self.auth.get_unsaved_user_from_token(token_data, token)

        self.assertEqual(user.email, 'john.doe2@example.net')
        self.assertEqual(user.customer_no, '5000167')
        self.assertEqual(user.first_name, 'John')
        self.assertEqual(user.last_name, 'Doe test')
        self.assertEqual(user.customer_name, 'Linara')
        self.assertEqual(user.title, 'MR')
        self.assertEqual(user.is_customer_admin, True)
        self.assertEqual(user.crm_contact_id, None)
        self.assertEqual(user.language, 'en')
        self.assertEqual(user.um_id, 13)


class FetchProfileTestCase(TestCase):
    def test_can_fetch_profile(self):

        # don't worry if this token is invalid. get a new one from my.solarlux.com (cookies tab)...
        token = 'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJsYXN0TmFtZSI6IkRvZSB0ZXN0IiwibGFuZ3VhZ2UiOiJlbiIsImlzQ3VzdG9tZXJBZG1pbiI6dHJ1ZSwiY3VzdG9tZXJOdW1iZXIiOiI1MDAwMTY3IiwidGl0bGUiOiJNUiIsImN1c3RvbWVyTmFtZSI6IkxpbmFyYSIsImF1dGhvcml0aWVzIjpbIlJPTEVfTkVXU19BUFAiLCJST0xFX1NQQUNFUyIsIlJPTEVfVklERU9TIiwiUk9MRV9HRU5FUkFMX05FV1MiLCJST0xFX1NPTEFSTFVYX0lOU0lERSIsIlJPTEVfVEVDSE5JQ0FMX0RPQ1MiLCJST0xFX1NPTEFSTFVYX0FETUlOIl0sImNsaWVudF9pZCI6Im15LWNsaWVudC13aXRoLXJlZ2lzdGVyZWQtcmVkaXJlY3QiLCJmaXJzdE5hbWUiOiJKb2huIiwic2NvcGUiOlsicmVhZCIsInRydXN0Il0sImlkIjoxMywiZXhwIjoxNTE1NTQ3OTY5LCJjcm1Db250YWN0SWQiOm51bGwsImVtYWlsIjoiam9obi5kb2UyQGV4YW1wbGUubmV0IiwianRpIjoiYjQ4NDQ0MTktYTFiNi00MGY4LWI3NTUtYTE3NzJiMjNjM2JiIn0.Gp7TR1McKE8KM-bPu1nvDMrKjvuC5Xe6CuVlPmiG-jUmlypgOl-_fonCo6OOozA90YXZPdq0n0FFt9a4oMjPXmQj00WMgOIvpH5yzRWNHArZQTH8vdAm4TU1iA2juzzz-WRQJz-khL4mvXDq24Ezcy7ZGYZ0sUZROGzWuEJi6IDcxpj659O1LWYpvXaVDocfPA5DFhrpbXQr3BePSgNAFrdm1sXOBXSRHNCeMbRVBPz1UDxRYGrXiuyU770uiVR8bbkOsS9QECgryHKpW6KO0q_xfYcEiuvOBAiaMHk7N4yALjYXq-NbRbonXVueLuKLd3lrI2_tdah96iOQ5JAh0A'

        response = requests.get('https://my.solarlux.com/apps/users/api/user/profile', headers={
            'Authorization': 'Bearer %s' % token
        }, timeout=0.5)

        self.assertEqual(response.status_code, 200)

        portal_config = response.json()


        user = self.auth.get_unsaved_user_from_token_and_portal_config(token, portal_config)

        self.assertEqual(user.avatar_attachment_id, 5)
        self.assertEqual(user.company_logo_attachment_id, None)



# Generated by Django 2.0.1 on 2018-02-08 22:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0009_auto_20180208_2347'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='myuser',
            name='customer_category',
        ),
        migrations.RemoveField(
            model_name='myuser',
            name='um_id',
        ),
        migrations.AlterField(
            model_name='myuser',
            name='customer_name',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='customer name'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='email',
            field=models.EmailField(blank=True, max_length=254, verbose_name='email address'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='first_name',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='first name'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='language',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='language'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='last_name',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='last name'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='password',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='password'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='title',
            field=models.CharField(blank=True, choices=[('MR', 'Mr.'), ('MRS', 'Mrs.')], max_length=255, null=True, verbose_name='title'),
        ),
    ]



# Generated by Django 2.0.1 on 2018-02-08 09:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0007_myuser_access_token'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='myuser',
            name='avatar_attachment_id',
        ),
        migrations.RemoveField(
            model_name='myuser',
            name='company_logo_attachment_id',
        ),
        migrations.AlterField(
            model_name='myuser',
            name='customer_category',
            field=models.CharField(blank=True, choices=[('QUALITY_PARTNER', 'quality partner'), ('DEALER', 'dealer'), ('ARCHITECT_PLANNER', 'architect/planner'), ('HOUSING_SOCIETY', 'housing society'), ('PRIVATE_CUSTOMER', 'private customer'), ('OTHERS', 'others'), ('INTERNAL', 'internal')], max_length=200, null=True, verbose_name='customer category'),
        ),
        migrations.AlterField(
            model_name='myuser',
            name='customer_no',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='customer no'),
        ),
    ]
